/**
 * Sistemas de Telecomunicacoes 2014/2015
 *
 * Daemon_udp.java
 *
 * Thread class that handles the reception of REGIST messages on the datagram
 * socket
 *
 * Created on March 22, 2015, 18:00
 *
 * @author Luis Bernardo
 */
package proxy;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;

public class Daemon_udp extends Thread {

    /**
     * Active flag
     */
    volatile boolean keepRunning = true;
    /**
     * GUI object
     */
    ProxyHttpd root;
    /**
     * Datagram socket object
     */
    DatagramSocket ds;

    /**
     * Constructor
     *
     * @param root GUI object
     * @param ds datagram socket
     */
    public Daemon_udp(ProxyHttpd root, DatagramSocket ds) {
        this.root = root;
        this.ds = ds;
    }

    @Override
    public void run() {
        byte[] buf = new byte[1500];    // Receive packets with up to 1500 bytes
        DatagramPacket dp = new DatagramPacket(buf, buf.length);
        try {
            while (keepRunning) {
                try {
                    ds.receive(dp);    // Wait for packets
                    ByteArrayInputStream BAis
                            = new ByteArrayInputStream(buf, 0, dp.getLength());
                    DataInputStream dis = new DataInputStream(BAis);
                    // process packet in ProxyHttpd object
                    root.receive_packet(dp, dis);
                } catch (SocketException se) {
                    if (keepRunning) {
                        root.Log("recv UDP SocketException : " + se + "\n");
                    }
                }
            }
        } catch (IOException e) {
            if (keepRunning) {
                root.Log("IO exception receiving data from socket : " + e);
            }
        }
    }

    /**
     * Stop the thread
     */
    public void stopRunning() {
        keepRunning = false;
    }
}
