/**
 * Sistemas de Telecomunicacoes 2014/2015
 *
 * PHttpThread.java
 *
 * Thread class that handles the (over)simplified HTTP protocol message exchange
 *
 * Created on March 22, 2015, 18:00
 *
 * @author Luis Bernardo
 */

package proxy;

import java.io.*;
import java.net.*;
import java.util.*;

public class PHttpThread extends Thread {
    /**
     * Reference to the GUI object
     */
    ProxyHttpd root;
    /**
     * Server socket where connects are accepted
     */
    ServerSocket ss;
    /**
     * Connection socket
     */
    Socket sock;

    /**
     * Creates a new instance of HttpThread
     * @param root
     * @param ss
     * @param sock
     */
    public PHttpThread(ProxyHttpd root, ServerSocket ss, Socket sock) {
        this.root = root;
        this.ss = ss;
        this.sock = sock;
    }

    /**
     * Send a "404 Not Found" error message to the brower
     *
     * @param pout Stream object associated to the sock's socket
     * @param servername Server name
     */
    public void return_error(PrintStream pout, String servername) {
        if (pout == null) {
            return;
        }
        pout.print("HTTP/1.0 404 Not Found\r\nServer: " + servername + "\r\n\r\n");
        // Prepares a web page with an error description
        pout.print("<HTML>\r\n");
        pout.print("<HEAD><TITLE>Not Found</TITLE></HEAD>\r\n");
        pout.print("<H1> Page not found </H1>\r\n");
        pout.print("</HTML>\r\n");
    }

    /**
     * Send a "301 Moved Permanently" message to the browser
     * 
     * @param pout Stream object associated to the sock's socket
     * @param servername Server name
     */
    private void return_redirect(PrintStream pout, String servername, ServerInfo srv) {
        if (pout == null) {
            return;
        }
        pout.print("HTTP/1.0 301 Moved Permanently\r\n");
        pout.print("Server: " + servername + "\r\n");
        /*
         * EXERCISE: Place here the code to construct the missing line, with the URL of the server with the format:
         *          Location: http://127.0.0.1:20000/path.html\r\n
         *  For a srv with an IP=127.0.0.1; port=20000; name= "path.html"
         * SUGGESTION: use the methods of class ServerInfo to get the information needed
         */
        pout.print("\r\n");
        // Prepares a web page with a description
        pout.print("<HTML>\r\n");
        pout.print("<HEAD><TITLE>Redirect</TITLE></HEAD>\r\n");
        pout.print("<H1> Redirected to "+srv.ip().getHostAddress()+":"+srv.port()+" </H1>\r\n");
        pout.print("</HTML>\r\n");
    }


    /**
     * Threads code that waits for a request and send the answer
     */
    @Override
    public void run() {
        PrintStream pout= null;
        try {
            BufferedReader in = new BufferedReader(
                    new InputStreamReader(sock.getInputStream(), "8859_1"));
            OutputStream out = sock.getOutputStream();
            pout = new PrintStream(out, false, "8859_1");
            String request = in.readLine(); // Read the first line 
            // The first request line has the format “GET /file.html HTTP/1.1”
            // This code uses the “StringTokenizer” class to separate the 3 components
            root.Log("Request= '" + request + '\n');
            StringTokenizer st = new StringTokenizer(request);
            if (st.countTokens() != 3) // If it does not have the 3 components isn't valid
            {
                return; // Invalid request
            }
            st.nextToken(); // Jumps the first token
            String file = st.nextToken(); // Gets the second token and ignores the third one
            file= file.substring(1);    // Removes the preceeding "/" from the filename
            if (file.equals(""))    // If "/" sets file to the default filename
                file = ProxyHttpd.HOMEFILENAME;

            /*
             * EXERCISE: Put here the code to verify if 'file' exists in the list
             *           and return a redirection code to the browser
             */
            root.Log("Proxy server always returns 404 Not Found error code\n");
            root.Log("Redirection not implemented yet\n");
            return_error(pout, ProxyHttpd.server_name);
            
        } catch (IOException e) {
            System.out.println("I/O error " + e);
        } catch (Exception e) {
            System.out.println("Error " + e);
        } finally {
            // This code is always run, even when there are exceptions
            try {
                sock.close(); // Closes the socket and all associated streams
            } catch (Exception e) { /* Ignore everything */ }
        }
    }

}
