/**
 * Sistemas de Telecomunicacoes 2014/2015
 *
 * ServerInfo.java
 *
 * Class used to store the information about a REGIST server
 *
 * Created on March 22, 2015, 18:00
 *
 * @author Luis Bernardo
 */
package proxy;

import java.net.InetAddress;
import java.util.Date;

public class ServerInfo {

    /**
     * The maximum number of miliseconds a serverinfo remains active
     */
    public final int VALIDITY = 15000;   // 15 seconds
    /**
     * File's name shared by the server
     */
    private final String name;
    /**
     * IP addressed of the server
     */
    private final InetAddress ip;
    /**
     * Port of the server
     */
    private final int port;
    /**
     * Server's registration date
     */
    private final Date date;

    /**
     * Constructor
     *
     * @param name file name shared
     * @param ip ip address
     * @param port port number
     */
    public ServerInfo(String name, InetAddress ip, int port) {
        this.name = name;
        this.ip = ip;
        this.port = port;
        this.date = new Date();
    }

    /**
     * Return the server's IP address
     *
     * @return IP address
     */
    public InetAddress ip() {
        return ip;
    }

    /**
     * Return the server's port number
     *
     * @return TCP port number
     */
    public int port() {
        return port;
    }

    /**
     * Return the filename
     *
     * @return filename
     */
    public String name() {
        return name;
    }

    /**
     * Return the registration date
     *
     * @return Date when REGIST was received
     */
    public Date date() {
        return date;
    }

    /**
     * Test if the server information is still valid
     *
     * @return true if is valid; false otherwise
     */
    public boolean valid() {
        return (new Date().getTime() - date.getTime()) < VALIDITY;
    }

    /**
     * toString converter
     *
     * @return the server information representation as a string
     */
    @Override
    public String toString() {
        if (ip != null) {
            return name() + "(" + ip.getHostAddress() + ":" + port + ")";
        } else {
            return "null:0";
        }
    }
}
